package eu.maiora

import eu.maiora.plugins.configureDatabases
import eu.maiora.plugins.*
import io.ktor.http.*
import io.ktor.server.application.*
import io.ktor.server.config.*
import io.ktor.server.engine.*
import io.ktor.server.netty.*
import io.ktor.server.plugins.callloging.*
import io.ktor.server.plugins.cors.routing.*
import java.io.FileInputStream
import java.util.*

fun main() {
    val properties  = loadConfig()
    val port = properties.getProperty("server.port").toInt()
    embeddedServer(Netty, port = port, host = "0.0.0.0") {
        module(properties)
    }.start(wait = true)
}

fun Application.module(configFile: Properties) {
    val config = ApplicationConfig("application.conf")
    val dbUrl = configFile.getProperty("ktor.database.url")
    val username = configFile.getProperty("ktor.database.username")
    val password = configFile.getProperty("ktor.database.password")
    val secret = config.property("ktor.jwt.secret").getString()
    configureDatabases(dbUrl, username, password)
    configureSecurity(secret)
    configureRouting(dbUrl, username, password)
    configureSerialization()

    install(CallLogging)
    install(CORS){
        anyHost()
        allowSameOrigin = true
        allowNonSimpleContentTypes = true
        allowHeader(HttpHeaders.AccessControlAllowOrigin)
        allowHeader(HttpHeaders.ContentType)
        allowMethod(HttpMethod.Options)
        allowMethod(HttpMethod.Put)
        allowMethod(HttpMethod.Patch)
        allowMethod(HttpMethod.Delete)
    }
}

fun loadConfig(): Properties {
    val properties = Properties()
    val inputStream = FileInputStream("/home/ristocloudadm/config.properties")
    properties.load(inputStream)
    return properties
}
